var searchData=
[
  ['_5f_5finit_5f_5f_31',['__init__',['../classecgdetectors_1_1Detectors.html#a35f3abf129faa91e9f1bab29e4f3bb63',1,'ecgdetectors.Detectors.__init__()'],['../classhrv_1_1HRV.html#a84adb37c163b24e0d7a41b8dfbcfd53a',1,'hrv.HRV.__init__()']]]
];
